#!/bin/bash
# Dialog frontend to manage SFTP users
# Copyright (C) 2013 Ferran Fontcuberta - All Rights Reserved
# Distributed under the terms of the GNU General Public License v3
# Last revised 2014/05/20

#SUID_COMMAND="/usr/local/bin/mangle_user_changes.sh"

# Disable Control C
trap '' 2
# Log STDERR
if [ ! -w /var/log/sftp-manager.log ];
then
    echo "Log file is not writeable!"
    exit -1
fi
exec 2>>/var/log/sftp-manager.log

# Internal variables#{{{
VERSION="0.2"
TEMP=$(mktemp)
TITLE=$"SFTP USER MANAGEMENT CONSOLE"
SFTP_MANAGER="${USERNAME_PREFIX}${SFTP_MANAGER}"
#TEXTDOMAINDIR=i18n
TEXTDOMAIN=manage-sftp-users
AUTHOR=$"Ferran Fontcuberta"
CONF_FILE="/etc/sftp-manager.conf"
#}}}
# Configuration parse{{{
if [ ! -r "$CONF_FILE" ]; then
    echo "Config file does not exist or unreadeable!"
    exit -1
fi
source "$CONF_FILE"
if [ -z $USERNAME_PREFIX ] || \
    [ -z $SFTP_MANAGER ] || \
    [ -z $CHROOT_PATH ] || \
    [ -z $LOG_FACILITY ] || \
    [ -z $LOG_LEVEL ];
then
    echo "Configuration failed."
    exit -1
fi
if [ "${CHROOT_PATH:0:1}" != '/' ];
then
    echo "CHROOT_PATH must start at root level"
    exit -1
fi
#}}}

# DIALOG FUNCTIONS#{{{
function _process { #{{{
	retval=$?
	parameter=$(cat $TEMP)
#	[[ $1 -eq 'main' ]] && [ $retval -eq 1 -o $retval -eq 255 ] && exit
	rm $TEMP 2>/dev/null
	[ $retval -eq 0 ] && OPT=$parameter || eval $1
} #}}}
function _search { #{{{
	OPT="null" ; T1=$1 ; T2=$2 ; shift 2
	#T1=$1 ; T2=$2 ; PARENT=$3 ; shift 3
	local i=0
	local -a opts
	for opt in "$@"
	do
		opts+=($i)
		opts+=("$opt")
		let i++
	done
	dialog --title "$T1" \
		--backtitle "$TITLE" \
		--menu "$T2" \
		0 0 0 \
		"${opts[@]}" \
	2> $TEMP
	_process
} #}}}
function _date { #{{{
	OPT="null" ; T1=$1 ; T2=$2 ; PARENT=$3 ; shift 3
	local TODAY=$(date +"%d %m %Y")
	dialog --title "$T1" \
		--backtitle "$TITLE" \
		--date-format '%Y-%m-%d' \
		--calendar "$T2" \
		0 0 \
		$TODAY \
	2> $TEMP
	_process $PARENT
} #}}}
function _input { #{{{
	OPT="null" ; T1=$1 ; T2=$2 ; PARENT=$3 ; shift 3
	INIT=$1
	dialog --title "$T1" \
		--backtitle "$TITLE" \
		--inputbox "$T2" \
		8 50 \
		"$INIT" \
	2> $TEMP
	_process $PARENT
} #}}}
function _msg { #{{{
	OPT="null" ; T1=$1 ; T2=$2 ; T3=$3 ; T4=$4 ; shift 4
	dialog --backtitle "$TITLE" --title "$T1" --msgbox "$T2" $T3 $T4

} #}}}
#}}}

function getUsers { #{{{
	printf "%s" "$(getent passwd|sed -ne '/^'"${SFTP_MANAGER}"'/d;/^'"${USERNAME_PREFIX}"'/ s/^\('"${USERNAME_PREFIX}"'[^:]*\):.*/\1/p')"
} #}}}
function getGroups { #{{{
	printf "%s" "$(getent group|sed -ne '/^'"${SFTP_MANAGER}"'/d;/^'"${USERNAME_PREFIX}"'/ s/^\([^:]*\):.*/\1/p')"
} #}}}

function createUser { #{{{
	#_input "" "User name" $FUNCNAME
	_input "" $"User name" _main
	#( [[ "$OPT" == '' ]] || [[ ! "$OPT" =~ ^[ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz]*$ ]] ) && $FUNCNAME
	if [[ "$OPT" == '' ]] || [[ ! "$OPT" =~ ^[\-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789]*$ ]] || [[ "$OPT" == "null" ]];
	then
		_msg "" $"ERROR" 0 0
		_main
	fi
	USERNAME="${USERNAME_PREFIX}${OPT}"
	if [[ "$USERNAME" == "$SFTP_MANAGER" ]];
	then
		_msg $"ERROR" $"This is the admin user" 0 0
		_main
	fi
	USERS=($(getUsers))
    # Esto no fallará si es el primero o el último??
	if [[ " ${USERS[@]} " =~ " ${USERNAME} " ]];
	then
		_msg "ERROR" $"This user already exists" 0 0
		_main
	fi

	PASS=$(/usr/bin/pwgen -s 15 1)
	GRPS=($(getGroups))
	[[ "${#GRPS[@]}" == "0" ]] && createGroup
	
	GRPS=($(getGroups))
        _search "" "" "Add new" "${GRPS[@]}"
	#[[ "$OPT" == "null" ]] && $FUNCNAME
	if [[ "$OPT" == "null" ]];
	then
		_msg "" $"ERROR" 0 0
		_main
	fi
	[[ "$OPT" == "0" ]] && createGroup
	GRPS=($(getGroups))
	GROUPNAME=${GRPS[ $(( $OPT -1 )) ]}
	#echo Creating user $USERNAME in group $GROUPNAME
	sudo useradd -g $GROUPNAME -d "${CHROOT_PATH}/$GROUPNAME/$USERNAME" -s /bin/sh $USERNAME
	sudo mkdir -p ${CHROOT_PATH}/$GROUPNAME/$USERNAME/upload
	sudo chmod 755 ${CHROOT_PATH}/$GROUPNAME/$USERNAME
	sudo chown root: ${CHROOT_PATH}/$GROUPNAME/$USERNAME
	sudo chown ${USERNAME}: ${CHROOT_PATH}/$GROUPNAME/$USERNAME/upload
#	_msg "" "/usr/sbin/useradd -g $GROUPNAME -d "/tmp/$USERNAME" -s /bin/sh $USERNAME" 0 0
	echo "${USERNAME}:${PASS}" | sudo /usr/sbin/chpasswd
	_msg "" $"User created, password: $PASS" 0 0
    sudo service ssh force-reload
	_main
} #}}}
function createGroup { #{{{
	local PARENT=$1
	_input "" $"Group name" _main
	#_input "" "Group name" $FUNCNAME
	#( [[ "$OPT" == '' ]] || [[ ! "$OPT" =~ ^[ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz]*$ ]] ) && $FUNCNAME
	if [[ "$OPT" == '' ]] || [[ ! "$OPT" =~ ^[\-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789]*$ ]] || [[ "$OPT" == "null" ]];
	then
		_msg "" $"ERROR" 0 0
		_main
	fi
	GROUPNAME="${USERNAME_PREFIX}${OPT}"
	GRPS=($(getGroups))
	if [[ " ${GRPS[@]} " =~ " $GROUPNAME " ]];
	then
		_msg $"ERROR" $"This group already exists" 0 0
		_main
	fi

	
	GRPID=$(getent group|awk '{ FS = ":" } {print $3}'|grep 1.....|sort -n|tail -n1)
	[ ! $GRPID ] && GRPID=100000
	GRPID=$(( $GRPID+1 ))
    # - internal-sftp -d %u (para que el home sea el nombre del username)
	sudo sh -c '/bin/echo -e "Match Group '${GROUPNAME}'\n\tChrootDirectory '${CHROOT_PATH}'/'${GROUPNAME}'/%u\n\tForceCommand internal-sftp -f '${LOG_FACILITY}' -l '${LOG_LEVEL}'" >> /etc/ssh/sshd_config'
	sudo groupadd -g $GRPID $GROUPNAME
	sudo mkdir -p ${CHROOT_PATH}/${GROUPNAME}
	_msg "" $"Group created" 0 0
	[[ "$1" != "" ]] && $1
} #}}}

function _main { #{{{

	dialog --title $"MAIN MENU" \
		--backtitle "$TITLE" \
		--menu "\n "$"This tool manages SFTP permissions." \
		15 50 10 \
		0 $"Manage users" \
		1 $"Manage groups" \
		2 $"About" \
	2> $TEMP
	_process
	case $OPT in
		0)
			USERS=($(getUsers))
			
			_search "" "" $"Add new" "${USERS[@]}"
			[[ "$OPT" == "null" ]] && $FUNCNAME
			[[ "$OPT" == "0" ]] && createUser
			_search "" "" $"Delete user"
			[[ "$OPT" == "null" ]] && $FUNCNAME
			if [[ "$OPT" == "0" ]];
			then
                local hack_for_USERGROUPS_ENAB
                local UNAME=${USERS[ $(( $OPT - 1 )) ]}
                if [ "$(groups $UNAME )" == "$UNAME" ];
                then
                    hack_for_USERGROUPS_ENAB=1
                fi
				sudo wipe -rfs $(getent passwd|grep "^$UNAME"|cut -d':' -f6)
				sudo userdel $UNAME
                if [ $hack_for_USERGROUPS_ENAB ] && [ $(getent group $UNAME &>/dev/null ; echo $?) -ne 0 ];
                then
                    sudo sed -i -e'/^Match Group '"${UNAME}"'$/,+2d' /etc/ssh/sshd_config
                fi
				
				_msg "" $"User deleted" 0 0
			fi			# OPT ?
			$FUNCNAME
			;;
		1)
			GRPS=($(getGroups))
			_search "" "" $"Add new" "${GRPS[@]}"
			[[ "$OPT" == "null" ]] && $FUNCNAME
			[[ "$OPT" == "0" ]] && createGroup $FUNCNAME
			GROUPNAME=${GRPS[ $(( $OPT - 1 )) ]}
			#groupMenu ${GRPS[ $(( $OPT - 1 )) ]}
			_search "" "" $"Delete group"
			[[ "$OPT" == "null" ]] && $FUNCNAME
			if [[ "$OPT" == "0" ]];
			then
				UGRPS=$(getent passwd|while IFS=: read name trash; do groups $name | grep -q -w "$GROUPNAME" && echo $name; done)
				if [[ "$UGRPS" == "" ]];
				then
					sudo groupdel $GROUPNAME
                    sudo sed -i -e'/^Match Group '"${GROUPNAME}"'$/,+2d' /etc/ssh/sshd_config
					_msg "" $"Group $GROUPNAME deleted" 0 0
				else
					_msg $"ERROR" $"This group contains users"":\n\n$UGRPS" 0 0
				fi
			fi
			$FUNCNAME
			;;
		2)
			_msg $"About" $"Written by $AUTHOR at innevis" 5 50
			$FUNCNAME
			exit
			;;
		*)
			_msg "" $"See ya!" 0 0
			exit
			;;
	esac
} #}}}
_main

# vim: set foldmethod=marker :
